@@ -2,7 +2,7 @@
  *  cache-file.c
  *  mod_musicindex
  *
- *  $Id: cache-file.c 780 2008-03-30 17:34:13Z varenet $
+ *  $Id: cache-file.c 823 2009-02-05 19:38:14Z varenet $
  *
  *  Created by Thibaut VARENE on Wed Feb 23 2005.
  *  Copyright (c) 2003-2007 Thibaut VARENE
@@ -18,7 +18,7 @@
  * Flat file cache management subsystem.
  *
  * @author Thibaut Varene
- * @version $Revision: 780 $
+ * @version $Revision: 823 $
  * @date 2003-2007
  *
  * @warning 'errno' seems not to be seen as volatile by the compiler, therefore we
@@ -53,7 +53,7 @@
 #include <string.h>	/* strerror() */
 #endif
 
-#define CACHE_VERS	11
+#define CACHE_VERS	12
 #define CACHE_NF	15
 
 #define CA_OK		0
@@ -169,7 +169,7 @@ static short file_cache_make_dir(request_rec *r, const char *const dirpath)
 	const char *tempdir = NULL;
 
 	do {	/* We build the path subdirs by subdirs, in a "mkdir -p" fashion */
-		tempdir = ap_pstrndup(r->pool, dirpath, (m + (l = strcspn(dirpath + m, "/"))));
+		tempdir = apr_pstrndup(r->pool, dirpath, (m + (l = strcspn(dirpath + m, "/"))));
 		m += l;
 
 		if (!l)
@@ -222,7 +222,7 @@ static void file_cache_remove_dir(request_rec *r, DIR *cachedir, const char *con
 				 * If it's a directory, we check that the "original" still exists.
 				 * If not, we remove it recursively.
 				 * Reminder: "errno == (EISDIR || EPERM)" doesn't work */
-				origdir = ap_pstrcat(r->pool, curdir, "/", cachedirent->d_name, NULL);
+				origdir = apr_pstrcat(r->pool, curdir, "/", cachedirent->d_name, NULL);
 				if (stat(origdir, &origdirstat)) {
 					if (rmdir(cachedirent->d_name)) {	/* stat() sets errno. We have to split */
 						if (errno == ENOTEMPTY) {			/* il est pas vide, bigre! */
@@ -420,13 +420,13 @@ static mu_ent *file_make_cache_entry(request_rec *r, apr_pool_t *pool, FILE *con
 				p = NULL;					/* hopefuly p allocs should be cleaned by apache */
 			}
 			else {
-				p->title = ap_pstrdup(r->pool, data_buffer->title);
+				p->title = apr_pstrdup(r->pool, data_buffer->title);
 				if (!ISBLANK(data_buffer->album))
-					p->album = ap_pstrdup(r->pool, data_buffer->album);
+					p->album = apr_pstrdup(r->pool, data_buffer->album);
 				if (!ISBLANK(data_buffer->artist))
-					p->artist = ap_pstrdup(r->pool, data_buffer->artist);
+					p->artist = apr_pstrdup(r->pool, data_buffer->artist);
 				if (!ISBLANK(data_buffer->genre))
-					p->genre = ap_pstrdup(r->pool, data_buffer->genre);
+					p->genre = apr_pstrdup(r->pool, data_buffer->genre);
 
 				/* We have to use that trick, fscanf won't work on char variables */
 				p->filetype = filetype;
@@ -520,7 +520,7 @@ int cache_file_setup(cmd_parms *cmd, const char *const setup_string, mu_config *
 
 	if (strncmp(biniou, setup_string, 7) == 0) {
 		conf->cache = &cache_backend_file;
-		conf->cache_setup = ap_pstrdup(cmd->pool, setup_string+6);
+		conf->cache_setup = apr_pstrdup(cmd->pool, setup_string+6);
 		return 0;
 	}
 
