@@ -2,11 +2,11 @@
  *  config.c
  *  mod_musicindex
  *
- *  $Id: config.c 754 2007-04-29 00:57:53Z varenet $
+ *  $Id: config.c 832 2009-02-14 19:56:14Z varenet $
  *
  *  Created by Thibaut VARENE on Thu Mar 20 2003.
  *  Copyright (c) 2003-2006 Regis BOUDIN
- *  Copyright (c) 2003-2005,2007 Thibaut VARENE
+ *  Copyright (c) 2003-2005,2007,2009 Thibaut VARENE
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License version 2.1,
@@ -20,12 +20,15 @@
  *
  * @author Regis Boudin
  * @author Thibaut Varene
- * @version $Revision: 754 $
+ * @version $Revision: 832 $
  * @date 2003-2007
+ * @date 2009
  *
  * That file takes care of the module configuration, either via setting some
  * known default values, and/or by getting them from environmental configuration
  * variables passed to apache in its config file.
+ *
+ * http://www.apachetutor.org/dev/config
  */
 
 #include "config.h"
@@ -90,7 +93,7 @@ static const char const default_css[] = "musicindex.css";	/**< Default CSS file
  */
 void *create_musicindex_config(apr_pool_t *p, char *dummy)
 {
-	mu_config *const new = (mu_config *)ap_pcalloc(p, sizeof(mu_config));
+	mu_config *const new = (mu_config *)apr_pcalloc(p, sizeof(mu_config));
 
 	memset(new->order, 0, sizeof(new->order));
 	memset(new->fields, 0, sizeof(new->fields));
@@ -123,89 +126,98 @@ void *create_musicindex_config(apr_pool_t *p, char *dummy)
  * Merge per-directory configurations.
  *
  * This function is supposed to merge two per-directory configurations. We
- * simply copy it from the parent directory.
+ * apply a simple inheritance pattern from the parent directory(ies).
+ *
+ * @note In the absence of a .htaccess in the hierarchy of folders being accessed,
+ * #basev contains a freshly created config structure (through
+ * #create_musicindex_config) and #addv contains another config struct created
+ * through #create_musicindex_config, with the configuration parameters modified
+ * by <Directory> directive (if any) set appropriately.
+ *
+ * @note In the presence of a .htaccess file in the directory being accessed,
+ * #basev contains the result of the merging of all "parent" configuration directives
+ * (<Directory> being the top ancestor, and any subsquent .htaccess being merged
+ * using #merge_musicindex_configs) up to the one in the current directory being
+ * accessed, which config options are set in #addv.
+ *
+ * @note In the presence of a .htaccess anywhere in the parent hierarchy of the
+ * directory being accessed @b BUT in said directory, #basev contains a pristine
+ * config structure (as per #create_musicindex_config) and #addv contains the
+ * result of merging all parent configuration directives (<Directory> being the
+ * top ancestor).
+ *
+ * Hence the logic of this function: if #addv contains init data (data setup in
+ * #create_musicindex_config), use #basev element. Otherwise, #addv contains
+ * modified config directive, use it instead.
  *
  * @param p Apache pool for memory allocation.
  * @param basev Pointer to main configuration structure.
  * @param addv Pointer to parent dir configuration structure,
- * 	is *apparently* equal to basev if no specific dir conf was set.
  *
  * @return The newly created configuration structure.
  */
 void *merge_musicindex_configs(apr_pool_t *p, void *basev, void *addv)
 {
-	mu_config *const new = (mu_config *)ap_pcalloc(p, sizeof(mu_config));
+	mu_config *const new = (mu_config *)apr_pcalloc(p, sizeof(mu_config));
 	mu_config *const base = (mu_config *)basev;
 	mu_config *const add = (mu_config *)addv;
 
-	if (memcmp(add->order, default_order, sizeof(default_order)))
-		memcpy(new->order, add->order, sizeof(add->order));
-	else
+	if (!memcmp(add->order, default_order, sizeof(default_order)))
 		memcpy(new->order, base->order, sizeof(base->order));
-
-	if (memcmp(add->fields, default_fields, sizeof(default_fields)))
-		memcpy(new->fields, add->fields, sizeof(add->fields));
 	else
+		memcpy(new->order, add->order, sizeof(add->order));
+
+	if (!memcmp(add->fields, default_fields, sizeof(default_fields)))
 		memcpy(new->fields, base->fields, sizeof(base->fields));
-
-	if (add->title != default_rootname)
-		new->title = ap_pstrdup(p, add->title);
-	else if (base->title != default_rootname)
-		new->title = ap_pstrdup(p, base->title);
 	else
-		new->title = default_rootname;
+		memcpy(new->fields, add->fields, sizeof(add->fields));
 
-	if (add->directory != default_directory)
-		new->directory = ap_pstrdup(p, add->directory);
-	else if (base->directory != default_directory)
-		new->directory = ap_pstrdup(p, base->directory);
+	if (add->title == default_rootname)
+		new->title = base->title;
 	else
-		new->directory = default_directory;
+		new->title = add->title;
 
-	if (add->css != default_css)
-		new->css = ap_pstrdup(p, add->css);
-	else if (base->css != default_css)
-		new->css = ap_pstrdup(p, base->css);
+	/* directory is currently non-modifiable */
+	new->directory = default_directory;
+
+	if (add->css == default_css)
+		new->css = base->css;
 	else
-		new->css = default_css;
+		new->css = add->css;
 
-	if ((add->cache != NULL) && (add->cache_setup != NULL)) {
-		new->cache = add->cache;
-		new->cache_setup = add->cache_setup;
-	} else if (base->cache != NULL) {
+	if (add->cache == NULL) {
 		new->cache = base->cache;
 		new->cache_setup = base->cache_setup;
 	} else {
-		new->cache = NULL;
-		new->cache_setup = NULL;
+		new->cache = add->cache;
+		new->cache_setup = add->cache_setup;
 	}
 
-	if (add->iceserver) /* new server defined ? */
-		new->iceserver = ap_pstrdup(p, add->iceserver);
-	else if (base->iceserver) /* previous sever defined ? */
-		new->iceserver = ap_pstrdup(p, base->iceserver);
-	else /* OK, no server defined... */
-		new->iceserver = NULL;
+	if (add->iceserver == NULL)
+		new->iceserver = base->iceserver;
+	else
+		new->iceserver = add->iceserver;
 
 	new->options = base->options | add->options;
-	new->options &= ~(add->options_not);
+	new->options &= ~(base->options_not | add->options_not);
+	
+	new->options_not = base->options_not | add->options_not;
+	new->options_not &= ~(base->options | add->options);
 
-	new->options_not = add->options_not;
-
-	if (add->cookie_life != CONF_COOKIE_LIFE)
-		new->cookie_life = add->cookie_life;
-	else
+	if (add->cookie_life == CONF_COOKIE_LIFE)
 		new->cookie_life = base->cookie_life;
-
-	if (add->rss_items != 0)
-		new->rss_items = add->rss_items;
 	else
+		new->cookie_life = add->cookie_life;
+
+	if (add->rss_items == 0)
 		new->rss_items = base->rss_items;
-
-	if (add->dir_per_line != CONF_DIRPERLINE)
-		new->dir_per_line = add->dir_per_line;
 	else
+		new->rss_items = add->rss_items;
+
+	if (add->dir_per_line == CONF_DIRPERLINE)
 		new->dir_per_line = base->dir_per_line;
+	else
+		new->dir_per_line = add->dir_per_line;
 
 	return new;
 }
@@ -237,7 +249,7 @@ static unsigned short sort_or_fields(cmd_parms *cmd, unsigned char *list, const
 	register unsigned short i = 0, j;
 
 	while (optstr[0] && (i < SB_MAX)) {
-		r = ap_getword_conf(cmd->pool, &optstr);
+		r = ap_getword_conf(cmd->temp_pool, &optstr);
 		for (j=0; options[j].string; j++) {
 			if (!strcasecmp(r, options[j].string)) {
 				list[i++] = options[j].value;
@@ -282,7 +294,7 @@ static const char *basic_config(cmd_parms *cmd, void *d, const char *optstr)
 	while (optstr[0] != '\0') {
 		int enable = 1;
 		unsigned short flag = 0;
-		r = ap_getword_conf(cmd->pool, &optstr);
+		r = ap_getword_conf(cmd->temp_pool, &optstr);
 
 		/* Should we enable or disable ? Is there a +/- prefix ? */
 		if ( *r == '-' ) {
@@ -370,7 +382,7 @@ static const char *set_page_title(cmd_parms *cmd, void *d, const char *optstr)
 	if ((optstr == NULL) || (optstr[0] == '\0'))
 		cfg->title = NULL;
 	else
-		cfg->title = ap_pstrdup(cmd->pool, optstr);
+		cfg->title = apr_pstrdup(cmd->pool, optstr);
 	return NULL;
 }
 
@@ -387,7 +399,7 @@ static const char *set_page_title(cmd_parms *cmd, void *d, const char *optstr)
 static const char *set_ice_server(cmd_parms *cmd, void *d, const char *optstr)
 {
 	mu_config *const cfg = (mu_config *)d;
-	cfg->iceserver = ap_pstrdup(cmd->pool, optstr);
+	cfg->iceserver = apr_pstrdup(cmd->pool, optstr);
 	return NULL;
 }
 
@@ -403,7 +415,7 @@ static const char *set_ice_server(cmd_parms *cmd, void *d, const char *optstr)
  */
 static const char *set_css_default(cmd_parms *cmd, void *d, const char *optstr)
 {
-	((mu_config *)d)->css = ap_pstrdup(cmd->pool, optstr);
+	((mu_config *)d)->css = apr_pstrdup(cmd->pool, optstr);
 	return NULL;
 }
 
@@ -426,6 +438,7 @@ static const char *set_cookie_life(cmd_parms *cmd, void *d, const char *optstr)
 
 /**
  * Sets the number of directories to display per line.
+ * Negative values will disable the "pretty folders" system.
  *
  * @param cmd I don't use it.
  * @param d Pointer to configuration structure.
@@ -435,7 +448,7 @@ static const char *set_cookie_life(cmd_parms *cmd, void *d, const char *optstr)
  */
 static const char *set_dirperline(cmd_parms *cmd, void *d, const char *optstr)
 {
-	((mu_config *)d)->dir_per_line = atoi(optstr);
+	((mu_config *)d)->dir_per_line = (atoi(optstr) ? atoi(optstr) : CONF_DIRPERLINE);
 	return NULL;
 }
 
@@ -466,8 +479,7 @@ static const char *set_display(cmd_parms *cmd, void *d, const char *optstr)
 	return NULL;
 }
 
-/* XXX if somebody puts a ACCESS_CONF|RSRC_CONF setting in .htaccess, the module
-seems to go totally foobar */
+/* XXX if somebody puts a ACCESS_CONF setting in .htaccess, Apache triggers a 500 */
 const command_rec musicindex_cmds[] = {
 	AP_INIT_RAW_ARGS("MusicIndex", basic_config, NULL, OR_INDEXES,
 		"can be : On/Off +/-Stream +/-Download +/-Search +/-Rss"),
@@ -475,17 +487,17 @@ const command_rec musicindex_cmds[] = {
 		"can be : title album artist track disc length bitrate filetype genre filename date uri"),
 	AP_INIT_RAW_ARGS("MusicFields", set_fields, NULL, OR_INDEXES,
 		"can be : title album artist track disc length bitrate filetype genre filename date"),
-	AP_INIT_RAW_ARGS("MusicIndexCache", set_cache_uri, NULL, ACCESS_CONF|RSRC_CONF,
+	AP_INIT_RAW_ARGS("MusicIndexCache", set_cache_uri, NULL, ACCESS_CONF,
 		"Set the cache configuration string"),
 	AP_INIT_RAW_ARGS("MusicPageTitle", set_page_title, NULL, OR_INDEXES,
 		"Set the root title of the page."),
-	AP_INIT_TAKE1("MusicIceServer", set_ice_server, NULL, ACCESS_CONF|RSRC_CONF,
+	AP_INIT_TAKE1("MusicIceServer", set_ice_server, NULL, ACCESS_CONF,
 		"Set the icecast server address : [server.domain.org]:8000"),
 	AP_INIT_TAKE1("MusicDefaultCss", set_css_default, NULL, OR_INDEXES,
 		"Set the default CSS file"),
-	AP_INIT_TAKE1("MusicCookieLife", set_cookie_life, NULL, ACCESS_CONF|RSRC_CONF,
+	AP_INIT_TAKE1("MusicCookieLife", set_cookie_life, NULL, ACCESS_CONF,
 		"Set the lifetime (in seconds) of the cookie sent for custom playlists"),
-	AP_INIT_TAKE1("MusicDefaultDisplay", set_display, NULL, ACCESS_CONF|RSRC_CONF,
+	AP_INIT_TAKE1("MusicDefaultDisplay", set_display, NULL, ACCESS_CONF,
 		"Set the default display returned by the module (HTML or RSS)"),
 	AP_INIT_TAKE1("MusicDirPerLine", set_dirperline, NULL, OR_INDEXES,
 		"Set the number of directories per line in the directory listing"),
