@@ -2,7 +2,7 @@
  *  html.c
  *  mod_musicindex
  *
- *  $Id: html.c 780 2008-03-30 17:34:13Z varenet $
+ *  $Id: html.c 826 2009-02-13 15:36:48Z varenet $
  *
  *  Created by Thibaut VARENE on Thu Mar 20 2003.
  *  Copyright (c) 2003-2006 Regis BOUDIN
@@ -20,7 +20,7 @@
  *
  * @author Regis Boudin
  * @author Thibaut Varene
- * @version $Revision: 780 $
+ * @version $Revision: 826 $
  * @date 2003-2008
  *
  * That file groups all functions related to HTML and assimilated
@@ -88,14 +88,14 @@ static const char *const Gcovericns[] = {
  * @return The total number of fields in a row.
  */
 static unsigned short
-list_songs(request_rec *r, const mu_ent *const p, const mu_config *const conf,
+list_songs(request_rec *r, const mu_pack *const pack, const mu_config *const conf,
 		const int customlist)
 {
-	const mu_ent *q = p;
+	const mu_ent *q;
 	register unsigned short fld = 0, j = 0;
 	const char *current = NULL, *new = NULL;
 	char *codeduri = NULL;
-	unsigned long tlength = 0, tsize = 0;
+	unsigned long tlength = 0, tsize = pack->fsize;
 
 	ap_rputs("  <tr class=\"title\">\n", r);
 
@@ -191,10 +191,7 @@ list_songs(request_rec *r, const mu_ent *const p, const mu_config *const conf,
 	current = "";
 	new = current;
 
-	for (q=p; q!= NULL; q=q->next) {
-		if (q->filetype < 0)
-			continue;
-
+	for (q=pack->fhead; q!= NULL; q=q->next) {
 		/* If we have a recursive search request we have some work to do */
 		if ((conf->search != NULL) && (customlist == 0) &&
 			(conf->options & MI_RECURSIVE)) {
@@ -226,13 +223,13 @@ list_songs(request_rec *r, const mu_ent *const p, const mu_config *const conf,
 		if ((customlist != 0) || (conf->options & (MI_ALLOWDWNLD | MI_ALLOWSTREAM | MI_ALLOWTARBALL)) ||
 			((conf->search) && (conf->options & MI_RECURSIVE))) {
 			/* this should save some memory */
-			codeduri = (char *)realloc(codeduri, 1 + ap_base64encode_len(strlen(q->uri)));
+			codeduri = (char *)realloc(codeduri, 1 + apr_base64_encode_len(strlen(q->uri)));
 
 			ap_rputs("   <td class=\"Select\">\n", r);
 
 			/* Display checkbox - no matter what if we have a customlist, only if stream/tarball allowed otherwise */
 			if (codeduri && (customlist || (conf->options & (MI_ALLOWSTREAM | MI_ALLOWTARBALL)))) {
-				ap_base64encode(codeduri, q->uri, strlen(q->uri));
+				apr_base64_encode(codeduri, q->uri, strlen(q->uri));
 				/* we base64 encode the uri to avoid nasty special chars problems */
 				ap_rvputs(r, "    <input type=\"checkbox\" name=\"file\" value=\"",
 					codeduri, "\" />\n", NULL);
@@ -312,17 +309,13 @@ list_songs(request_rec *r, const mu_ent *const p, const mu_config *const conf,
 					ap_rprintf(r, "   <td class=\"Album\">%s</td>\n", q->album ? ap_escape_html(r->pool, q->album) : "");
 					break;
 				case SB_FILETYPE:
-					if (q->filetype >= 0)
 						ap_rvputs(r, "   <td class=\"Filetype\">", filetype[q->filetype % FT_MAX].nametype, "</td>\n", NULL);
-					else
-						ap_rputs("   <td></td>\n", r);
 					break;
 				case SB_GENRE:
 					ap_rprintf(r, "   <td class=\"Genre\">%s</td>\n", q->genre ? ap_escape_html(r->pool, q->genre) : "");
 					break;
 				case SB_SIZE:
 					ap_rprintf(r, "   <td class=\"Size\"><acronym title=\"MB\">%ld.%ld</acronym></td>\n", q->size>>20, (q->size>>10)%1024/100);
-					tsize += q->size;
 					break;
 				case SB_FILENAME:
 					ap_rvputs(r, "   <td class=\"Filename\">", ap_escape_html(r->pool, q->filename), "</td>\n", NULL);
@@ -395,12 +388,12 @@ static void send_url(request_rec *r, const char *const uri, const char *const co
 
 		if (REQUEST_USER(r)) {
 			/* grab the auth credentials base64 encoded - apparently only for auth_basic */
-			const char *const auth = ap_table_get(r->headers_in, "Authorization");
+			const char *const auth = apr_table_get(r->headers_in, "Authorization");
 			if (auth) {
 				bufcoded = strrchr(auth, ' ');
 				bufcoded++;
-				decoded = (char *)ap_palloc(r->pool, 1 + ap_base64decode_len(bufcoded));
-				l = ap_base64decode(decoded, bufcoded);
+				decoded = (char *)apr_palloc(r->pool, 1 + apr_base64_decode_len(bufcoded));
+				l = apr_base64_decode(decoded, bufcoded);
 				strncat(prefix, decoded, l);	/* we have "user:pass" */
 			}
 			strcat(prefix, "@");
@@ -504,8 +497,8 @@ void send_head(request_rec *r, const mu_config *const conf)
 
 	/* RSS link */
 	if (conf->rss_items > 0) {
-		ap_rputs(" <link rel=\"alternate\" type=\"application/rss+xml\" "
-			"title=\"Latest titles\" href=\"?action=RSS\" />\n", r);
+		ap_rvputs(r, " <link rel=\"alternate\" type=\"application/rss+xml\" "
+			"title=\"", _("Latest titles"), "\" href=\"?action=RSS\" />\n", NULL);
 		ap_rputs(" <link rel=\"alternate\" type=\"application/rss+xml\" "
 			"title=\"Podcast\" href=\"?action=podcast\" />\n", r);
 	}
@@ -524,12 +517,12 @@ void send_head(request_rec *r, const mu_config *const conf)
 		"  <img alt=\"Dir\" src=\"", r);
 
 	for (i = 0; Gcovericns[i]; i++) {
-		if (access(ap_pstrcat(r->pool, r->filename, "/", Gcovericns[i], NULL), R_OK) == 0) {
+		if (access(apr_pstrcat(r->pool, r->filename, "/", Gcovericns[i], NULL), R_OK) == 0) {
 			ap_rputs(Gcovericns[i], r);
 			i = -1;
 			break;
 		}
-		else if (access(ap_pstrcat(r->pool, r->filename, "/.", Gcovericns[i], NULL), R_OK) == 0) {
+		else if (access(apr_pstrcat(r->pool, r->filename, "/.", Gcovericns[i], NULL), R_OK) == 0) {
 			ap_rvputs(r, ".", Gcovericns[i], NULL);
 			i = -1;
 			break;
@@ -546,7 +539,7 @@ void send_head(request_rec *r, const mu_config *const conf)
 	ap_rputs(" <div id=\"maintitle\">\n"
 		"  <h1>\n", r);
 
-	uri = ap_pstrdup(r->pool, r->uri);
+	uri = apr_pstrdup(r->pool, r->uri);
 
 	/* XXX Could use some doc... :P */
 	for (u = uri; *u != '\0'; u++) {
@@ -676,7 +669,7 @@ void send_directories(request_rec *r, const mu_pack *const pack, const mu_config
 			"    <a href=\"", ap_escape_html(r->pool, ap_escape_uri(r->pool, q->file)), "\">",
 			ap_escape_html(r->pool, q->title), "</a>", NULL);
 
-		if (ENABLE_PRETTY_FOLDERS) {
+		if (conf->dir_per_line > 0) {
 			/* show various useful links when needed */
 			ap_rputs("<br />\n", r);
 			if (q->flags & EF_ALLOWSTREAM) {
@@ -709,7 +702,7 @@ void send_directories(request_rec *r, const mu_pack *const pack, const mu_config
 			"  </td>\n",r);
 
 		/* end the directories row */
-		if (dircnt == conf->dir_per_line) {
+		if (dircnt == abs(conf->dir_per_line)) {
 			dircnt = 0;
 			ap_rputs(" </tr>\n", r);
 		}
@@ -735,7 +728,6 @@ void send_directories(request_rec *r, const mu_pack *const pack, const mu_config
  */
 void send_tracks(request_rec *r, const mu_pack *const pack, const mu_config *const conf)
 {
-	const mu_ent *q = pack->head;
 	register unsigned short nb = 0;
 	unsigned long fnb = pack->filenb;
 	const unsigned char *const restrict order = conf->order;
@@ -753,7 +745,7 @@ void send_tracks(request_rec *r, const mu_pack *const pack, const mu_config *con
 	ap_rvputs(r, "</h2>\n\n<form method=\"post\" action=\"", ap_escape_html(r->pool, ap_escape_uri(r->pool, r->uri)), "\" "
 		"enctype=\"application/x-www-form-urlencoded\" id=\"tracks\">\n <table>\n", NULL);
 
-	nb = list_songs(r, q, conf, 0);
+	nb = list_songs(r, pack, conf, 0);
 
 	ap_rputs(" </table>\n", r);
 
@@ -815,10 +807,9 @@ void send_tracks(request_rec *r, const mu_pack *const pack, const mu_config *con
  */
 void send_customlist(request_rec *r, const mu_pack *const pack, const mu_config *const conf)
 {
-	const mu_ent *q = pack->head;
 	register unsigned short nb = pack->filenb;
 
-	if (!q)
+	if (!pack->fhead)
 		return;
 
 	ap_rputs("<!-- begin custom -->\n"
@@ -829,7 +820,7 @@ void send_customlist(request_rec *r, const mu_pack *const pack, const mu_config
 	ap_rvputs(r, " <form method=\"post\" action=\"", ap_escape_html(r->pool, ap_escape_uri(r->pool, r->uri)), "\" "
 		"enctype=\"application/x-www-form-urlencoded\" id=\"custom\">\n  <table>\n", NULL);
 
-	list_songs(r, q, conf, 1);
+	list_songs(r, pack, conf, 1);
 
 	ap_rvputs(r, "  </table>\n"
 		"  <div>\n   <button type=\"submit\" name=\"action\" value=\"RemoveFromPlaylist\" class=\"playlist\">",
@@ -859,7 +850,7 @@ void send_customlist(request_rec *r, const mu_pack *const pack, const mu_config
  */
 void send_playlist(request_rec *r, const mu_pack *const pack, const mu_config *const conf)
 {
-	const mu_ent *q = pack->head;
+	const mu_ent *q = pack->fhead;
 
 	if (!q)
 		return;
@@ -895,7 +886,7 @@ void send_playlist(request_rec *r, const mu_pack *const pack, const mu_config *c
  */
 void send_rss(request_rec *r, const mu_pack *const pack, const mu_config *const conf)
 {
-	const mu_ent *q = pack->head;
+	const mu_ent *q = pack->fhead;
 	char date_buf[32];
 	struct tm time_buf;
 	unsigned short limit = conf->rss_items;
@@ -911,14 +902,19 @@ void send_rss(request_rec *r, const mu_pack *const pack, const mu_config *const
 
 	ap_rvputs(r, "version=\"2.0\">\n"
 		" <channel>\n"
-		"  <title>RSS Feed for ", ap_escape_html(r->pool, r->uri), "</title>\n"
+		"  <title>", _("RSS Feed for "), ap_escape_html(r->pool, r->uri), "</title>\n"
 		"  <link>", NULL);
 	send_url(r, r->uri, NULL, conf, 1);
 
-	ap_rprintf(r, "</link>\n  <description>%d most recent songs from %s</description>\n", conf->rss_items, ap_escape_html(r->pool, r->uri));
-
-	if ((conf->options & MI_PODCAST) != 0)
-		ap_rprintf(r, "  <itunes:summary>%d most recent songs from %s</itunes:summary>\n", conf->rss_items, ap_escape_html(r->pool, r->uri));
+	ap_rputs("</link>\n  <description>", r);
+	ap_rprintf(r, _("%d most recent songs from %s"), conf->rss_items, ap_escape_html(r->pool, r->uri));
+	ap_rputs("</description>\n", r);
+	
+	if ((conf->options & MI_PODCAST) != 0) {
+		ap_rputs("  <itunes:summary>", r);
+		ap_rprintf(r, _("%d most recent songs from %s"), conf->rss_items, ap_escape_html(r->pool, r->uri));
+		ap_rputs("</itunes:summary>\n", r);
+	}
 
 	ap_rputs("  <generator>mod_musicindex/" MUSIC_VERSION_STRING "</generator>\n"
 		"  <docs>http://backend.userland.com/rss</docs>\n"
@@ -926,9 +922,6 @@ void send_rss(request_rec *r, const mu_pack *const pack, const mu_config *const
 		r);
 
 	for (; limit && q; q = q->next) {
-		if (q->filetype < 0)	/* XXX try to get rid of this test */
-			continue;
-
 		ap_rvputs(r, "  <item>\n"
 			"   <title>", ap_escape_html(r->pool, q->title), "</title>\n", NULL);
 
@@ -957,21 +950,28 @@ void send_rss(request_rec *r, const mu_pack *const pack, const mu_config *const
 			}
 		}
 
-		ap_rputs("   <description>\n    Artist | Album | Track | Disc | Length | Genre | Bitrate | Freq | Type | Size\n", r);
+		ap_rvputs(r, "   <description>\n    ", _("Artist"), " | ", _("Album"), " | ", _("Track"), " | ", _("Disc"),
+					" | ", _("Length"), " | ", _("Genre"), " | ", _("Bitrate"), " | ", _("Freq"), " | ", _("Filetype"),
+					" | ", _("Size"), "<br />\n", NULL);
 		ap_rprintf(r, "    %s | %s | ", q->artist ? ap_escape_html(r->pool, q->artist) : "",
 				q->album ? ap_escape_html(r->pool, q->album) : "");
 		if (q->track)
-			ap_rprintf(r, "%u | ", q->track);
+			ap_rprintf(r, "%u", q->track);
+		ap_rputs(" | ", r);
 		if (q->posn)
-			ap_rprintf(r, "%u | ", q->posn);
+			ap_rprintf(r, "%u", q->posn);
+		ap_rputs(" | ", r);
 		if (q->length)
-			ap_rprintf(r, "%u:%.2u | ", q->length / 60, q->length % 60);
+			ap_rprintf(r, "%u:%.2u", q->length / 60, q->length % 60);
+		ap_rputs(" | ", r);
 		ap_rprintf(r, "%s | ", q->genre ? ap_escape_html(r->pool, q->genre) : "");
 		if (q->bitrate)
-			ap_rprintf(r, "%lu %s | ", q->bitrate >> 10, (q->flags & EF_VBR) ? "VBR" : "");
+			ap_rprintf(r, "%lu %s", q->bitrate >> 10, (q->flags & EF_VBR) ? "VBR" : "");
+		ap_rputs(" | ", r);
 		if (q->freq)
-			ap_rprintf(r, "%u | ", q->freq);
-		ap_rprintf(r, "%s | %lu\n", q->filetype ? filetype[q->filetype % FT_MAX].nametype : "", q->size);
+			ap_rprintf(r, "%u", q->freq);
+		ap_rputs(" | ", r);
+		ap_rprintf(r, "%s | %lu\n", filetype[q->filetype % FT_MAX].nametype, q->size);
 		ap_rputs("   </description>\n"
 			"  </item>\n", r);
 
@@ -1006,12 +1006,12 @@ void send_randomdir(request_rec *r, const mu_config *const conf)
 	DIR *dir;
 	struct dirent *dirent;
 
-	filename = ap_pstrdup(r->pool, r->filename);
-	uri = ap_pstrdup(r->pool, r->uri);
+	filename = apr_pstrdup(r->pool, r->filename);
+	uri = apr_pstrdup(r->pool, r->uri);
 	
 	while (1) {
 		/* XXX this is ugly */
-		filename = ap_pstrcat(r->pool, filename, "/", NULL);
+		filename = apr_pstrcat(r->pool, filename, "/", NULL);
 	
 		/* open the current directory for scanning */
 		dir = opendir(filename);
@@ -1023,6 +1023,13 @@ void send_randomdir(request_rec *r, const mu_config *const conf)
 			if ((dirent->d_name[0] != '.') && (access(r->filename, R_OK|X_OK) == 0))
 				nb++;
 
+		/* if nb == 0, we've got an empty directory. Don't go any further */
+		if (!nb) {
+			closedir(dir);
+			apr_table_setn(r->headers_out, "Location", ap_construct_url(r->pool, ap_escape_uri(r->pool, uri), r));
+			return;
+		}
+		
 		/* once it's done, go back to the begining of the dir stream */
 		rewinddir(dir);
 		
@@ -1034,10 +1041,10 @@ void send_randomdir(request_rec *r, const mu_config *const conf)
 		while((i <= test) && (dirent = readdir(dir)))
 			if ((dirent->d_name[0] != '.') && (access(r->filename, R_OK|X_OK) == 0))
 				i++;
-
+		
 		/* construct a new filename with the current dirent */
-		nextfile = ap_pstrcat(r->pool, filename, dirent->d_name, NULL);
-		nexturi = ap_pstrcat(r->pool, uri, dirent->d_name, NULL);
+		nextfile = apr_pstrcat(r->pool, filename, dirent->d_name, NULL);
+		nexturi = apr_pstrcat(r->pool, uri, dirent->d_name, NULL);
 
 		/* done with the dir listing, close it */
 		closedir(dir);
@@ -1045,13 +1052,13 @@ void send_randomdir(request_rec *r, const mu_config *const conf)
 		/* if the new filename is a directory, start the loop again, else, break */
 		if (!ap_is_directory(r->pool, nextfile)) {
 			/* XXX this is probably not good either */
-			ap_table_setn(r->headers_out, "Location", ap_construct_url(r->pool, ap_escape_uri(r->pool, uri), r));
+			apr_table_setn(r->headers_out, "Location", ap_construct_url(r->pool, ap_escape_uri(r->pool, uri), r));
 			return;
 		}
 		else {
 			/* XXX gack! */
 			filename = nextfile;
-			uri = ap_pstrcat(r->pool, nexturi, "/", NULL);
+			uri = apr_pstrcat(r->pool, nexturi, "/", NULL);
 		}
 	}
 }
@@ -1078,8 +1085,8 @@ void send_foot(request_rec *r, const mu_config *const conf)
 		" <div id=\"valid\">\n", r);
 	/* if we have local icons, use them */
 	/* valid xhtml 1.1 */
-	if (subreq && !access(ap_pstrcat(r->pool, subreq->filename, "/valid-xhtml11", NULL), R_OK)) {
-		icn = ap_pstrcat(r->pool, conf->directory, "/valid-xhtml11", NULL);
+	if (subreq && !access(apr_pstrcat(r->pool, subreq->filename, "/valid-xhtml11", NULL), R_OK)) {
+		icn = apr_pstrcat(r->pool, conf->directory, "/valid-xhtml11", NULL);
 		ap_rvputs(r, "  <img src=\"", icn, "\" alt=\"Valid XHTML 1.1!\" />\n", NULL);
 	}
 	else
@@ -1088,8 +1095,8 @@ void send_foot(request_rec *r, const mu_config *const conf)
 			 "    alt=\"Valid XHTML 1.1!\" height=\"31\" width=\"88\" />\n"
 			 "  </a>\n", r);
 	/* valid CSS */
-	if (subreq && !access(ap_pstrcat(r->pool, subreq->filename, "/vcss", NULL), R_OK)) {
-		icn = ap_pstrcat(r->pool, conf->directory, "/vcss", NULL);
+	if (subreq && !access(apr_pstrcat(r->pool, subreq->filename, "/vcss", NULL), R_OK)) {
+		icn = apr_pstrcat(r->pool, conf->directory, "/vcss", NULL);
 		ap_rvputs(r, "  <img src=\"", icn, "\" alt=\"Valid CSS!\" />\n", NULL);
 	}
 	else
@@ -1098,8 +1105,8 @@ void send_foot(request_rec *r, const mu_config *const conf)
 			 "    alt=\"Valid CSS!\" height=\"31\" width=\"88\" />\n"
 			 "  </a>\n", r);
 	/* Valid RSS */
-	if (subreq && !access(ap_pstrcat(r->pool, subreq->filename, "/loveRss.gif", NULL), R_OK)) {
-		icn = ap_pstrcat(r->pool, conf->directory, "/loveRss.gif", NULL);
+	if (subreq && !access(apr_pstrcat(r->pool, subreq->filename, "/loveRss.gif", NULL), R_OK)) {
+		icn = apr_pstrcat(r->pool, conf->directory, "/loveRss.gif", NULL);
 		ap_rvputs(r, "  <img src=\"", icn, "\" alt=\"Valid RSS!\" />\n", NULL);
 	}
 	else
