@@ -2,11 +2,11 @@
  *  mod_musicindex.c
  *  mod_musicindex
  *
- *  $Id: mod_musicindex.c 769 2007-08-22 12:43:06Z varenet $
+ *  $Id: mod_musicindex.c 823 2009-02-05 19:38:14Z varenet $
  *
  *  Created by Regis BOUDIN on Sat Dec 28 2002.
  *  Copyright (c) 2002-2006 Regis BOUDIN
- *  Copyright (c) 2002-2005,2007 Thibaut VARENE
+ *  Copyright (c) 2002-2005,2007-2008 Thibaut VARENE
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License version 2.1,
@@ -107,9 +107,8 @@
  * This file is the core of the module. It contains Apache's mandatory stuff.
  *
  * @author Regis Boudin
- * @version $Revision: 769 $
- * @date 2002-2005
- * @date 2007
+ * @version $Revision: 823 $
+ * @date 2002-2008
  *
  * http://httpd.apache.org/docs-2.0/developer/API.html
  * http://httpd.apache.org/dev/apidoc/
@@ -122,7 +121,7 @@
  * @todo (low pri whishlist) On the fly tag rewriting or metadata support.
  * @todo prepare the possibility to generate ices/mpd playlists.
  * @todo enforce strict type policy.
- * @todo get rid of str*cmp() whenever possible and eventually implement "likely()".
+ * @todo get rid of str*cmp() whenever possible.
  * @todo review the randomdir implementation.
  *
  * @todo redesign cache integration to speed it up:
@@ -182,21 +181,44 @@ static const char *handlers[] = {
 #endif
 
 #ifdef ENABLE_OUTPUT_ARCHIVE
-extern void send_tarball(request_rec *, const mu_pack *const, const mu_config *const);
+extern void send_tarball(request_rec *, const mu_pack *const);
+extern ssize_t tarball_size(request_rec *, const mu_pack *const);
 #endif
 
+/**
+ * Check whether we're serving MS IE.
+ *
+ * @param r Apache request_rec
+ *
+ * @return 1 if client is MSIE, 0 otherwise
+ */
+static int is_msie_user_agent(request_rec *r)
+{
+    const char* user_agent = NULL;
+
+    user_agent = apr_table_get(r->headers_in, "User-Agent");
+    return strstr(user_agent, "MSIE") && 1;
+}
+
+/**
+ * This is where the magic happens.
+ */
 static int handle_musicindex(request_rec *r)
 {
 	/* Get the module configuration */
 	mu_config *const conf = (mu_config *)ap_get_module_config(r->per_dir_config, &musicindex_module);
 	mu_pack master_pack = {
 		.head = NULL,
+		.fhead = NULL,
 		.dirnb = 0,
 		.filenb = 0,
+		.fsize = 0,
 	},	custom_pack = {
 		.head = NULL,
+		.fhead = NULL,
 		.dirnb = 0,
 		.filenb = 0,
+		.fsize = 0,
 	};
 
 	/* if the the module is not active, or if request is not a GET or a POST,
@@ -227,13 +249,13 @@ static int handle_musicindex(request_rec *r)
 	if (r->uri[0] == '\0' || r->uri[strlen(r->uri) - 1] != '/') {
 		char *file;
 		if (r->args != NULL)
-			file = ap_pstrcat(r->pool, ap_escape_uri(r->pool, r->uri),
+			file = apr_pstrcat(r->pool, ap_escape_uri(r->pool, r->uri),
 						"/", "?", r->args, NULL);
 		else
-			file = ap_pstrcat(r->pool, ap_escape_uri(r->pool, r->uri),
+			file = apr_pstrcat(r->pool, ap_escape_uri(r->pool, r->uri),
 						"/", NULL);
 
-		ap_table_setn(r->headers_out, "Location", ap_construct_url(r->pool, file, r));
+		apr_table_setn(r->headers_out, "Location", ap_construct_url(r->pool, file, r));
 		return HTTP_MOVED_PERMANENTLY;
 	}
 
@@ -284,15 +306,27 @@ static int handle_musicindex(request_rec *r)
 	/* Depending on the request type, set the HTTP headers. By default we
 	   return a web page. */
 	if (conf->options & MI_STREAM) {
+		char content_disposition[64] = "";
+			if (is_msie_user_agent(r))	/* this helps IE dealing with the data */
+				strcat(content_disposition, "attachment; ");
+
+		strcat(content_disposition, "filename=\"playlist.m3u\"");
+
 		ap_set_content_type(r, "audio/x-mpegurl");
-		ap_table_setn(r->headers_out, "Content-Disposition",
-			"filename = \"playlist.m3u\"");
+		apr_table_setn(r->headers_out, "Content-Disposition", content_disposition);
 	}
+#ifdef ENABLE_OUTPUT_ARCHIVE
 	else if (conf->options & MI_TARBALL) {
+		const ssize_t tballsize = tarball_size(r, &master_pack);
 		ap_set_content_type(r, "application/x-tar");
-		ap_table_setn(r->headers_out, "Content-Disposition",
+		apr_table_setn(r->headers_out, "Content-Disposition",
 			"filename = \"playlist.tar\"");
+		/* send the size of the archive so that browsers can put a nice
+		   progress bar */
+		apr_table_setn(r->headers_out, "Content-Length",
+			apr_psprintf(r->pool, "%ld", tballsize));
 	}
+#endif
 	else if (conf->options & MI_RSS) {
 		ap_set_content_type(r, "text/xml; charset=\"utf-8\"");
 	}
@@ -301,7 +335,7 @@ static int handle_musicindex(request_rec *r)
 
 		/* If we have a cookie string, send it */
 		if (conf->custom_list != NULL)
-			ap_table_setn(r->headers_out, "Set-Cookie", conf->custom_list);
+			apr_table_setn(r->headers_out, "Set-Cookie", conf->custom_list);
 	}
 
 	ap_send_http_header(r);
@@ -315,7 +349,7 @@ static int handle_musicindex(request_rec *r)
 		send_rss(r, &master_pack, conf);
 #ifdef ENABLE_OUTPUT_ARCHIVE
 	else if (conf->options & MI_TARBALL)
-		send_tarball(r, &master_pack, conf);
+		send_tarball(r, &master_pack);
 #endif
 	else {
 		send_head(r, conf);
@@ -383,12 +417,13 @@ static int handle_musicfile(request_rec *r)
 	if ((conf->options & MI_ALLOWSTREAM) && (strcmp(r->args, "stream") == 0)) {
 		mu_pack master_pack = {
 			.head = NULL,
+			.fhead = NULL,
 			.dirnb = 0,
 			.filenb = 0,
 		};
 
 		ap_set_content_type(r, "audio/x-mpegurl");
-		ap_table_setn(r->headers_out, "Content-Disposition",
+		apr_table_setn(r->headers_out, "Content-Disposition",
 			"filename = \"playlist.m3u\"");
 
 		ap_send_http_header(r);
@@ -397,6 +432,7 @@ static int handle_musicfile(request_rec *r)
 			return OK;
 
 		make_music_entry(r, r->pool, &master_pack, conf, NULL, MI_ALLOWSTREAM);
+		master_pack.fhead = master_pack.head;
 		send_playlist(r, &master_pack, conf);
 
 		return OK;
