@@ -2,11 +2,11 @@
  *  mod_musicindex.h
  *  mod_musicindex
  *
- *  $Id: mod_musicindex.h 780 2008-03-30 17:34:13Z varenet $
+ *  $Id: mod_musicindex.h 824 2009-02-05 19:41:31Z varenet $
  *
  *  Created by Thibaut VARENE on Thu Mar 20 2003.
  *  Copyright (c) 2003-2006 Regis BOUDIN
- *  Copyright (c) 2003-2007 Thibaut VARENE
+ *  Copyright (c) 2003-2009 Thibaut VARENE
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License version 2.1,
@@ -58,6 +58,18 @@
  #define TRUE !FALSE
 #endif
 
+#ifndef C_HAS_ATTRIBUTE
+ #define __attribute__(x)	/* NOTHING */
+#endif
+
+#ifdef C_HAS_BUILTIN_EXPECT
+ #define likely(x)       __builtin_expect(!!(x), 1)
+ #define unlikely(x)     __builtin_expect(!!(x), 0)
+#else
+ #define likely(x)	(x)
+ #define unlikely(x)	(x)
+#endif
+
 /* a very common macro */
 #define ARRAY_SIZE(x) (sizeof(x) / sizeof(x[0]))
 
@@ -201,22 +213,23 @@ enum {
  #define ap_sub_req_lookup_uri(p, r, ptr) ap_sub_req_lookup_uri(p, r)
  #define REQUEST_USER(rec) (rec->connection->user)
 
+ #define apr_pool_destroy ap_destroy_pool
+
+ #define apr_table_get ap_table_get
+ #define apr_table_setn ap_table_setn
+ #define apr_palloc ap_palloc
+ #define apr_pcalloc ap_pcalloc
+ #define apr_pstrdup ap_pstrdup
+ #define apr_pstrndup ap_pstrndup
+ #define apr_pstrcat ap_pstrcat
+ #define apr_psprintf ap_psprintf
+ #define apr_base64_encode_len ap_base64encode_len
+ #define apr_base64_encode ap_base64encode
+ #define apr_base64_decode_len ap_base64decode_len
+ #define apr_base64_decode ap_base64decode
+
 #else
 
- #define ap_base64encode apr_base64_encode
- #define ap_table_get apr_table_get
- #define ap_palloc apr_palloc
- #define ap_destroy_pool apr_pool_destroy
- #define ap_table_setn apr_table_setn
- #define ap_pstrcat apr_pstrcat
- #define ap_base64encode_len apr_base64_encode_len
- #define ap_cpystrn apr_cpystrn
- #define ap_pcalloc apr_pcalloc
- #define ap_pstrdup apr_pstrdup
- #define ap_psprintf apr_psprintf
- #define ap_pstrndup apr_pstrndup
- #define ap_base64decode_len apr_base64_decode_len
- #define ap_base64decode apr_base64_decode
 
  #if defined(AP20)
   /* nada */
@@ -254,11 +267,11 @@ typedef struct mu_ent_names {
  * Special care has been taken toward typing to avoid memory waste.
  */
 typedef struct mu_ent {
+	const struct mu_ent *next;
 	signed char filetype;	/**< file type, associated to a display string and a mimetype string */
 	unsigned char flags;	/**< flags associated with the file */
 	unsigned char track;	/**< track number */
 	unsigned char posn;	/**< "part of a set" number, that is, disc number */
-	unsigned short date;	/**< date (year actually) */
 	unsigned short freq;	/**< samplerate frequency */
 	unsigned short length;	/**< length in seconds */
 	unsigned long bitrate;	/**< bitrate in bits per second */
@@ -271,8 +284,8 @@ typedef struct mu_ent {
 	const char *artist;	/**< artist (if any) */
 	const char *title;	/**< title (as shown on the webpages) */
 	const char *genre;	/**< genre (type of music - if any) */
-	const struct mu_ent *next;
-} mu_ent;
+	unsigned short date;	/**< date (year actually) */
+} /* __attribute__((packed)) */ mu_ent;
 
 /**
  * Pack of music entries.
@@ -281,7 +294,9 @@ typedef struct mu_ent {
  */
 typedef struct {
 	const mu_ent *head;	/**< pointer to the head of mu_ent list */
+	const mu_ent *fhead;	/**< pointer to the head of the songs list (if any) */
 	unsigned long filenb;	/**< number of files in the list */
+	unsigned long fsize;	/**< total size of the pack (files) in bytes */
 	unsigned short dirnb;	/**< number of directories in the list */
 } mu_pack;
 
@@ -351,7 +366,7 @@ struct mu_config {
 	const cache_backend *cache;
 	void *cache_setup;
 
-	unsigned short dir_per_line;	/**< number of directories to display per line */
+	signed short dir_per_line;	/**< number of directories to display per line. If <0 disable "pretty folders" */
 };
 
 
