@@ -2,10 +2,10 @@
  *  output-tarball.c
  *  mod_musicindex
  *
- *  $Id: output-tarball.c 739 2007-04-22 22:25:05Z varenet $
+ *  $Id: output-tarball.c 812 2008-12-21 01:34:29Z varenet $
  *
  *  Created by Thibaut VARENE on Mon Jun 26 2006.
- *  Copyright (c) 2006-2007 Thibaut VARENE
+ *  Copyright (c) 2006-2008 Thibaut VARENE
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License version 2.1,
@@ -18,11 +18,10 @@
  * Tarballs generation
  *
  * @author Thibaut Varene
- * @version $Revision: 739 $
- * @date 2006-2007
+ * @version $Revision: 812 $
+ * @date 2006-2008
  *
- * This file generates and send on the fly a tarball of the list given as
- * parameter, using libarchive.
+ * This file handles on the fly tarball generation and associated operations
  *
  * @todo use apr_file I/O
  */
@@ -49,6 +48,10 @@
 #include <stdio.h>	/* fopen... */
 
 #define RWBUFSIZE	8192	/**< Size of our r/w buffer */
+#define QSKIP_CHECKS(q)	(!(q->flags & EF_ALLOWTARBALL))
+#define ARCHIVE_SET_FORMAT(a)	\
+	archive_write_set_compression_none(a); \
+	archive_write_set_format_ustar(a)
 
 /** Holds our client data for libarchive */
 struct cdata {
@@ -69,28 +72,21 @@ wrapwrite(struct archive *a, void *client_data, void *buff, size_t n)
 	return (ap_rwrite(buff, n, mydata->r));
 }
 
-static int
-wrapclose(struct archive *a, void *client_data)
-{
-	return (0);
-}
-
 /**
  * Sends tarball.
  *
  * @param r Apache request_rec struct to handle connection details.
  * @param pack A pack of music entries.
- * @param conf MusicIndex configuration paramaters struct.
  *
  * It is a known "feature" that when tar'ing a custom playlist, the generated
  * tarball will contain the full path as published on the webserver, including
- * '/'. We could get rid of the leading '/' if needed, but there's one good reason
- * not to put all the files at the root of one big tarball: in case there are name
- * doublons.
+ * '/'. We could get rid of the leading '/' if needed, but there's one good
+ * reason not to put all the files at the root of one big tarball: in case there
+ * are name doublons.
  */
-void send_tarball(request_rec *r, const mu_pack *const pack, const mu_config *const conf)
+void send_tarball(request_rec *r, const mu_pack *const pack)
 {
-	const mu_ent *q = pack->head;
+	const mu_ent *q = pack->fhead;
 	struct cdata *mydata;
 	struct archive *a;
 	struct archive_entry *entry;
@@ -105,28 +101,22 @@ void send_tarball(request_rec *r, const mu_pack *const pack, const mu_config *co
 	if (!(mydata = malloc(sizeof(struct cdata))))
 		return;
 
-	if (!(a = archive_write_new())) {
-		free(mydata);
-		return;
-	}
+	if (!(a = archive_write_new()))
+		goto fail1;
+	
+	if (!(entry = archive_entry_new()))
+		goto fail2;
 
 	mydata->r = r;
-	archive_write_set_compression_none(a);
-	archive_write_set_format_ustar(a);
-	archive_write_open(a, mydata, wrapopen, wrapwrite, wrapclose);
+	ARCHIVE_SET_FORMAT(a);
+	archive_write_open(a, mydata, wrapopen, wrapwrite, NULL);
 
 	for (; q; q = q->next) {
-		if (q->filetype < 0)
+		if (unlikely(QSKIP_CHECKS(q)))
 			continue;
 
-		/* do not allow downloading if the file flags deny it */
-		if (!(q->flags & EF_ALLOWTARBALL))
-			continue;
-
-		if (!(entry = archive_entry_new()))
-			break;
-
 		stat(q->filename, &st);
+		archive_entry_clear(entry);
 		archive_entry_copy_stat(entry, &st);
 		archive_entry_set_pathname(entry, q->file);
 		archive_write_header(a, entry);
@@ -138,10 +128,76 @@ void send_tarball(request_rec *r, const mu_pack *const pack, const mu_config *co
 			archive_write_data(a, buff, len);
 			len = fread(buff, 1, sizeof(buff), file);
 		}
-		archive_entry_free(entry);
 		fclose(file);
 	}
 
+	archive_entry_free(entry);
+fail2:
 	archive_write_finish(a);
+fail1:
 	free(mydata);
 }
+
+static ssize_t
+wrapwritesize(struct archive *a, void *client_data, void *buff, size_t n)
+{
+	ssize_t *mysize = client_data;
+	*mysize += n;
+	return n;
+}
+
+/**
+ * Compute tarball size
+ *
+ * @param r Apache request_rec struct to handle connection details.
+ * @param pack A pack of music entries.
+ *
+ * This function must be kept coherent with send_tarball()
+ */
+ssize_t
+tarball_size(request_rec *r, const mu_pack *const pack)
+{
+	const mu_ent *q = pack->fhead;
+	ssize_t tbsize = -1;
+	struct archive *a;
+	struct archive_entry *ae;
+	struct stat st;
+	static char buff[RWBUFSIZE];
+	int len;
+	off_t fsize;
+
+	if (!q)
+		goto fail1;
+
+	if (!(a = archive_write_new()))
+		goto fail1;
+
+	if (!(ae = archive_entry_new()))
+		goto fail2;
+	
+	ARCHIVE_SET_FORMAT(a);
+	archive_write_open(a, &tbsize, NULL, wrapwritesize, NULL);
+
+	for (; q; q = q->next) {
+		if (QSKIP_CHECKS(q))
+			continue;
+
+		stat(q->filename, &st);
+		archive_entry_clear(ae);
+		archive_entry_copy_stat(ae, &st);
+		archive_entry_set_pathname(ae, q->file);
+		archive_write_header(a, ae);
+
+		fsize = q->size;
+		len = (likely(fsize > sizeof(buff)) ? sizeof(buff) : fsize);
+		while (len > 0) {
+			fsize -= archive_write_data(a, buff, len);
+			len = (likely(fsize > sizeof(buff)) ? sizeof(buff) : fsize);
+		}
+	}
+	archive_entry_free(ae);
+fail2:
+	archive_write_finish(a);
+fail1:
+	return tbsize;
+}
