@@ -2,7 +2,7 @@
  *  playlist-flac.c
  *  mod_musicindex
  *
- *  $Id: playlist-flac.c 754 2007-04-29 00:57:53Z varenet $
+ *  $Id: playlist-flac.c 823 2009-02-05 19:38:14Z varenet $
  *
  *  Created by Regis BOUDIN on Thu Jun 30 2004.
  *  Copyright (c) 2004 Regis BOUDIN
@@ -20,7 +20,7 @@
  *
  * @author Regis Boudin
  * @author Thibaut Varene
- * @version $Revision: 754 $
+ * @version $Revision: 823 $
  * @date 2004-2005
  * @date 2007
  *
@@ -128,11 +128,11 @@ mu_ent *make_flac_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
 			vc = &block->data.vorbis_comment;
 			for(i = 0; i < vc->num_comments; i++) {
 				if (!strncasecmp((char *)vc->comments[i].entry, "album=", 6))
-					p->album = ap_pstrndup(pool, (char *)vc->comments[i].entry+6, vc->comments[i].length-6);
+					p->album = apr_pstrndup(pool, (char *)vc->comments[i].entry+6, vc->comments[i].length-6);
 				else if (!strncasecmp((char *)vc->comments[i].entry, "artist=", 7))
-					p->artist = ap_pstrndup(pool, (char *)vc->comments[i].entry+7, vc->comments[i].length-7);
+					p->artist = apr_pstrndup(pool, (char *)vc->comments[i].entry+7, vc->comments[i].length-7);
 				else if (!strncasecmp((char *)vc->comments[i].entry, "title=", 6))
-					p->title = ap_pstrndup(pool, (char *)vc->comments[i].entry+6, vc->comments[i].length-6);
+					p->title = apr_pstrndup(pool, (char *)vc->comments[i].entry+6, vc->comments[i].length-6);
 				else if (!strncasecmp((char *)vc->comments[i].entry, "tracknumber=", 12))
 					p->track = atoi((char *)vc->comments[i].entry+12);
 				else if (!strncasecmp((char *)vc->comments[i].entry, "discnumber=", 11))
