@@ -2,7 +2,7 @@
  *  playlist-mp4.c
  *  mod_musicindex
  *
- *  $Id: playlist-mp4.c 754 2007-04-29 00:57:53Z varenet $
+ *  $Id: playlist-mp4.c 828 2009-02-14 16:33:54Z varenet $
  *
  *  Created by Thibaut VARENE on Tue Sep  2 2004.
  *  Copyright (c) 2004-2005,2007 Thibaut VARENE
@@ -18,7 +18,7 @@
  * MP4 files management system.
  *
  * @author Thibaut Varene
- * @version $Revision: 754 $
+ * @version $Revision: 828 $
  * @date 2004-2005
  * @date 2007
  *
@@ -31,7 +31,7 @@
 #include "playlist.h"
 #include "playlist-mp4.h"
 
-#include <mp4.h>        /* libmp4v2 */
+#include <mp4v2/mp4v2.h>        /* libmp4v2 */
 
 /**
  * Checks for valid MP4 filename extension.
@@ -126,17 +126,17 @@ mu_ent *make_mp4_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
 			}
 		
 			if (MP4GetMetadataArtist(mp4file, &temp)) {
-				p->artist = ap_pstrdup(pool, temp);
+				p->artist = apr_pstrdup(pool, temp);
 				free(temp);
 			}
 			
 			if (MP4GetMetadataName(mp4file, &temp)) {
-				p->title = ap_pstrdup(pool, temp);
+				p->title = apr_pstrdup(pool, temp);
 				free(temp);
 			}
 			
 			if (MP4GetMetadataAlbum(mp4file, &temp)) {
-				p->album = ap_pstrdup(pool, temp);
+				p->album = apr_pstrdup(pool, temp);
 				free(temp);
 			}
 			
@@ -146,7 +146,7 @@ mu_ent *make_mp4_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
 			}
 		
 			if (MP4GetMetadataGenre(mp4file, &temp)) {
-				p->genre = ap_pstrdup(pool, temp);
+				p->genre = apr_pstrdup(pool, temp);
 				free(temp);
 			}
 		
