@@ -2,11 +2,11 @@
  *  playlist.c
  *  mod_musicindex
  *
- *  $Id: playlist.c 780 2008-03-30 17:34:13Z varenet $
+ *  $Id: playlist.c 823 2009-02-05 19:38:14Z varenet $
  *
  *  Created by Thibaut VARENE on Thu Mar 20 2003.
  *  Copyright (c) 2003-2004 Regis BOUDIN
- *  Copyright (c) 2003-2007 Thibaut VARENE
+ *  Copyright (c) 2003-2008 Thibaut VARENE
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License version 2.1,
@@ -20,8 +20,8 @@
  *
  * @author Regis Boudin
  * @author Thibaut Varene
- * @version $Revision: 780 $
- * @date 2003-2007
+ * @version $Revision: 823 $
+ * @date 2003-2008
  *
  * This is where the magic takes place. Here are the functions in charge of
  * analyzing the content of the folders requested by the client, and of
@@ -101,7 +101,7 @@ static inline mu_ent *make_no_entry(request_rec *r, apr_pool_t *pool, FILE *cons
 static mu_ent *make_cache_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
 	const mu_config *const conf, mu_ent_names *const names)
 {
-	if (conf->cache && conf->cache->make_entry) {
+	if (conf->cache && likely(conf->cache->make_entry)) {
 		mu_ent *result = conf->cache->make_entry(r, pool, in, conf, names);
 		if (result) {
 			result->flags |= EF_INCACHE;
@@ -285,6 +285,8 @@ go_through_directory(request_rec *r, const mu_config *const conf, const unsigned
  *
  * @note It'd be very nice to be able to make this function tail recursive,
  * unfortunately it looks like it's going to be rather troublesome to do so.
+ * @note This function will NOT and should NEVER muck with mu_pack.fhead. This
+ * should be taken care of by whoever calls it.
  *
  * @param pool Pool
  * @param r Apache request_rec struct to handle log writings (debugging)
@@ -309,7 +311,7 @@ void make_music_entry(request_rec *r, apr_pool_t *pool, mu_pack *const pack,
 		so no assertion can be made on their respective length */
 		if ((strlen(r->filename) >= MAX_STRING) || (strlen(r->parsed_uri.path) >= MAX_STRING))
 			return;
-		names = ap_palloc(pool, sizeof(mu_ent_names));
+		names = apr_palloc(pool, sizeof(mu_ent_names));
 		strcpy(names->filename, r->filename);
 		strcpy(names->uri, r->parsed_uri.path);
 	}
@@ -340,7 +342,7 @@ void make_music_entry(request_rec *r, apr_pool_t *pool, mu_pack *const pack,
 			*uri++ = '/';
 		*uri = '\0';
 
-		if (ENABLE_PRETTY_FOLDERS) {
+		if (conf->dir_per_line > 0) {
 			subreq = ap_sub_req_lookup_uri(names->uri, r, NULL);
 
 			soptions &= ~(MI_ALLOWFETCH|MI_ALLOWRSS);
@@ -365,7 +367,7 @@ void make_music_entry(request_rec *r, apr_pool_t *pool, mu_pack *const pack,
 			unsigned short	fn_max = MAX_STRING;
 			unsigned short	uri_max = MAX_STRING;
 
-			if (!ENABLE_PRETTY_FOLDERS) {
+			if (conf->dir_per_line < 0) {
 				subreq = ap_sub_req_lookup_uri(names->uri, r, NULL);
 
 				soptions &= ~(MI_ALLOWFETCH|MI_ALLOWRSS);
@@ -437,7 +439,7 @@ void make_music_entry(request_rec *r, apr_pool_t *pool, mu_pack *const pack,
 
 	p->next = pack->head; /* Add it to the linked list */
 
-	p->uri = ap_pstrdup(pool, names->uri); /* The absolute URI will be useful at some point */
+	p->uri = apr_pstrdup(pool, names->uri); /* The absolute URI will be useful at some point */
 	p->filename = p->file = p->uri; /* Add the relative filename */
 
 	/* We remove the path prefix if any. We have to treat files differently
@@ -464,7 +466,7 @@ void make_music_entry(request_rec *r, apr_pool_t *pool, mu_pack *const pack,
 	if ((soptions & MI_CUSTOM) == 0)
 		p->file += strlen(r->parsed_uri.path);  /* remove the path before the filename except for custom playlists */
 	if (conf->options & MI_TARBALL)	/* if the user asked for a tarball, we're going to need the absolute path to access the file */
-		p->filename = ap_pstrdup(pool, names->filename);
+		p->filename = apr_pstrdup(pool, names->filename);
 
 	/* In the case of directories, we want to have a pretty printed version
 	 * of the directory name, for display and sorting reasons (trailing '/'
@@ -476,7 +478,7 @@ void make_music_entry(request_rec *r, apr_pool_t *pool, mu_pack *const pack,
 		p->title = p->filename;
 #else
 		/* Copy the name removing file extension (if any) and changing '_' to ' ' */
-		char *const stitle = ap_pstrndup(pool, p->filename,
+		char *const stitle = apr_pstrndup(pool, p->filename,
 					((p->filetype >= 0) && strrchr(p->filename, '.')) ?	/* only strip ext if !dir */
 					(strrchr(p->filename, '.') - p->filename) : /* <=> strlen(p->file) - strlen(strrchr(p->file, '.')) */
 					strlen(p->filename));
@@ -502,9 +504,12 @@ void make_music_entry(request_rec *r, apr_pool_t *pool, mu_pack *const pack,
 
 	/* We put that here so that we do not create cache files for unhandled file types,
 	 * and it's faster to retrieve p->title than to process it again. */
-	if (conf->cache && (!(p->flags & EF_INCACHE)) && conf->cache->write)
+	if (conf->cache && (!(p->flags & EF_INCACHE)) && likely((!(conf->options & MI_QUICKPL)) && conf->cache->write))
 		conf->cache->write(r, p, conf, names);
-	
+
+	/* sum the total size of the files in the pack */
+	pack->fsize += p->size;
+
 	/* Here we go if we have an ongoing search request, and the file we're
 	 * looking at wasn't found in the cache. We then fall back to a basic strcasestr()
 	 * check, returning only the entries matching the search criteria */
@@ -571,11 +576,11 @@ void build_custom_list(request_rec *r, mu_pack *const pack, const mu_config *con
 		else if (direct)
 			continue;
 
-		decodeduri = (char *)realloc(decodeduri, 1 + ap_base64decode_len(p));
+		decodeduri = (char *)realloc(decodeduri, 1 + apr_base64_decode_len(p));
 		if (!decodeduri)
 			return;
 		
-		ap_base64decode(decodeduri, p);
+		apr_base64_decode(decodeduri, p);
 		/* decodeduri is unescaped. We need to escape it before
 		   passing a new request to the server */
 		p = ap_escape_uri(r->pool, decodeduri);
@@ -606,7 +611,7 @@ void build_custom_list(request_rec *r, mu_pack *const pack, const mu_config *con
 		custom->next = result;
 		result = custom;
 	}
-	pack->head = result;
+	pack->fhead = result;
 
 	return;
 }
