@@ -2,7 +2,7 @@
  *  playlist.h
  *  mod_musicindex
  *
- *  $Id: playlist.h 742 2007-04-23 13:15:36Z varenet $
+ *  $Id: playlist.h 823 2009-02-05 19:38:14Z varenet $
  *
  *  Created by Thibaut VARENE on Thu Mar 20 2003.
  *  Copyright (c) 2003-2004 Regis BOUDIN
@@ -48,7 +48,7 @@ struct ftype {
 };
 extern const struct ftype const filetype[];
 
-#define NEW_ENT(pool) ((mu_ent *)ap_pcalloc(pool, sizeof(mu_ent)))
+#define NEW_ENT(pool) ((mu_ent *)apr_pcalloc(pool, sizeof(mu_ent)))
 
 void make_music_entry(request_rec *, apr_pool_t *,
 	mu_pack *const, const mu_config *const, mu_ent_names *, unsigned long);
